@@ -17,18 +17,27 @@
 # <http://www.gnu.org/licenses/> for a copy of the GNU General Public License
 # and <http://www.apache.org/licenses/LICENSE-2.0> for a copy of the Apache
 # License, Version 2.0.
+#
 
 """Implementation of merge-base following the approach of git."""
 
-from collections.abc import Iterator
+__all__ = [
+    "WorkList",
+    "can_fast_forward",
+    "find_merge_base",
+    "find_octopus_base",
+    "independent",
+]
+
+from collections.abc import Callable, Iterator, Mapping, Sequence
 from heapq import heappop, heappush
-from typing import TYPE_CHECKING, Any, Callable, Generic, Optional, TypeVar
+from typing import TYPE_CHECKING, Generic, TypeVar
 
 if TYPE_CHECKING:
     from .repo import BaseRepo
 
 from .lru_cache import LRUCache
-from .objects import ObjectID
+from .objects import Commit, ObjectID
 
 T = TypeVar("T")
 
@@ -52,7 +61,7 @@ class WorkList(Generic[T]):
         dt, cmt = item
         heappush(self.pq, (-dt, cmt))
 
-    def get(self) -> Optional[tuple[int, T]]:
+    def get(self) -> tuple[int, T] | None:
         """Get the highest priority item from the work list.
 
         Returns:
@@ -77,10 +86,10 @@ class WorkList(Generic[T]):
 def _find_lcas(
     lookup_parents: Callable[[ObjectID], list[ObjectID]],
     c1: ObjectID,
-    c2s: list[ObjectID],
+    c2s: Sequence[ObjectID],
     lookup_stamp: Callable[[ObjectID], int],
     min_stamp: int = 0,
-    shallows: Optional[set[ObjectID]] = None,
+    shallows: set[ObjectID] | None = None,
 ) -> list[ObjectID]:
     """Find lowest common ancestors between commits.
 
@@ -96,7 +105,7 @@ def _find_lcas(
         List of lowest common ancestor commit IDs
     """
     cands = []
-    cstates = {}
+    cstates: dict[ObjectID, int] = {}
 
     # Flags to Record State
     _ANC_OF_1 = 1  # ancestor of commit 1
@@ -104,7 +113,9 @@ def _find_lcas(
     _DNC = 4  # Do Not Consider
     _LCA = 8  # potential LCA (Lowest Common Ancestor)
 
-    def _has_candidates(wlst: WorkList[ObjectID], cstates: dict[ObjectID, int]) -> bool:
+    def _has_candidates(
+        wlst: WorkList[ObjectID], cstates: Mapping[ObjectID, int]
+    ) -> bool:
         """Check if there are any candidate commits in the work list.
 
         Args:
@@ -122,7 +133,7 @@ def _find_lcas(
 
     # initialize the working list states with ancestry info
     # note possibility of c1 being one of c2s should be handled
-    wlst: WorkList[bytes] = WorkList()
+    wlst: WorkList[ObjectID] = WorkList()
     cstates[c1] = _ANC_OF_1
     try:
         wlst.add((lookup_stamp(c1), c1))
@@ -203,7 +214,7 @@ def _find_lcas(
 
 
 # actual git sorts these based on commit times
-def find_merge_base(repo: "BaseRepo", commit_ids: list[ObjectID]) -> list[ObjectID]:
+def find_merge_base(repo: "BaseRepo", commit_ids: Sequence[ObjectID]) -> list[ObjectID]:
     """Find lowest common ancestors of commit_ids[0] and *any* of commits_ids[1:].
 
     Args:
@@ -212,13 +223,17 @@ def find_merge_base(repo: "BaseRepo", commit_ids: list[ObjectID]) -> list[Object
     Returns:
       list of lowest common ancestor commit_ids
     """
-    cmtcache: LRUCache[ObjectID, Any] = LRUCache(max_cache=128)
+    cmtcache: LRUCache[ObjectID, Commit] = LRUCache(max_cache=128)
     parents_provider = repo.parents_provider()
 
     def lookup_stamp(cmtid: ObjectID) -> int:
         if cmtid not in cmtcache:
-            cmtcache[cmtid] = repo.object_store[cmtid]
-        return cmtcache[cmtid].commit_time
+            obj = repo.object_store[cmtid]
+            assert isinstance(obj, Commit)
+            cmtcache[cmtid] = obj
+        commit_time = cmtcache[cmtid].commit_time
+        assert isinstance(commit_time, int)
+        return commit_time
 
     def lookup_parents(cmtid: ObjectID) -> list[ObjectID]:
         commit = None
@@ -232,7 +247,7 @@ def find_merge_base(repo: "BaseRepo", commit_ids: list[ObjectID]) -> list[Object
     c1 = commit_ids[0]
     if not len(commit_ids) > 1:
         return [c1]
-    c2s = commit_ids[1:]
+    c2s = list(commit_ids[1:])
     if c1 in c2s:
         return [c1]
     lcas = _find_lcas(
@@ -241,7 +256,9 @@ def find_merge_base(repo: "BaseRepo", commit_ids: list[ObjectID]) -> list[Object
     return lcas
 
 
-def find_octopus_base(repo: "BaseRepo", commit_ids: list[ObjectID]) -> list[ObjectID]:
+def find_octopus_base(
+    repo: "BaseRepo", commit_ids: Sequence[ObjectID]
+) -> list[ObjectID]:
     """Find lowest common ancestors of *all* provided commit_ids.
 
     Args:
@@ -250,13 +267,17 @@ def find_octopus_base(repo: "BaseRepo", commit_ids: list[ObjectID]) -> list[Obje
     Returns:
       list of lowest common ancestor commit_ids
     """
-    cmtcache: LRUCache[ObjectID, Any] = LRUCache(max_cache=128)
+    cmtcache: LRUCache[ObjectID, Commit] = LRUCache(max_cache=128)
     parents_provider = repo.parents_provider()
 
     def lookup_stamp(cmtid: ObjectID) -> int:
         if cmtid not in cmtcache:
-            cmtcache[cmtid] = repo.object_store[cmtid]
-        return cmtcache[cmtid].commit_time
+            obj = repo.object_store[cmtid]
+            assert isinstance(obj, Commit)
+            cmtcache[cmtid] = obj
+        commit_time = cmtcache[cmtid].commit_time
+        assert isinstance(commit_time, int)
+        return commit_time
 
     def lookup_parents(cmtid: ObjectID) -> list[ObjectID]:
         commit = None
@@ -286,7 +307,7 @@ def find_octopus_base(repo: "BaseRepo", commit_ids: list[ObjectID]) -> list[Obje
     return lcas
 
 
-def can_fast_forward(repo: "BaseRepo", c1: bytes, c2: bytes) -> bool:
+def can_fast_forward(repo: "BaseRepo", c1: ObjectID, c2: ObjectID) -> bool:
     """Is it possible to fast-forward from c1 to c2?
 
     Args:
@@ -294,13 +315,17 @@ def can_fast_forward(repo: "BaseRepo", c1: bytes, c2: bytes) -> bool:
       c1: Commit id for first commit
       c2: Commit id for second commit
     """
-    cmtcache: LRUCache[ObjectID, Any] = LRUCache(max_cache=128)
+    cmtcache: LRUCache[ObjectID, Commit] = LRUCache(max_cache=128)
     parents_provider = repo.parents_provider()
 
     def lookup_stamp(cmtid: ObjectID) -> int:
         if cmtid not in cmtcache:
-            cmtcache[cmtid] = repo.object_store[cmtid]
-        return cmtcache[cmtid].commit_time
+            obj = repo.object_store[cmtid]
+            assert isinstance(obj, Commit)
+            cmtcache[cmtid] = obj
+        commit_time = cmtcache[cmtid].commit_time
+        assert isinstance(commit_time, int)
+        return commit_time
 
     def lookup_parents(cmtid: ObjectID) -> list[ObjectID]:
         commit = None
@@ -334,3 +359,38 @@ def can_fast_forward(repo: "BaseRepo", c1: bytes, c2: bytes) -> bool:
         shallows=parents_provider.shallows,
     )
     return lcas == [c1]
+
+
+def independent(repo: "BaseRepo", commit_ids: Sequence[ObjectID]) -> list[ObjectID]:
+    """Filter commits to only those that are not reachable from others.
+
+    Args:
+      repo: Repository object
+      commit_ids: list of commit ids to filter
+
+    Returns:
+      list of commit ids that are not ancestors of any other commits in the list
+    """
+    if not commit_ids:
+        return []
+    if len(commit_ids) == 1:
+        return list(commit_ids)
+
+    # Filter out commits that are ancestors of other commits
+    independent_commits = []
+    for i, commit_id in enumerate(commit_ids):
+        is_independent = True
+        # Check if this commit is an ancestor of any other commit
+        for j, other_id in enumerate(commit_ids):
+            if i == j:
+                continue
+            # If merge base of (commit_id, other_id) is commit_id,
+            # then commit_id is an ancestor of other_id
+            merge_bases = find_merge_base(repo, [commit_id, other_id])
+            if merge_bases == [commit_id]:
+                is_independent = False
+                break
+        if is_independent:
+            independent_commits.append(commit_id)
+
+    return independent_commits
