@@ -123,6 +123,7 @@ class BlackboxTestCase(TestCase):
 
 def self_test_suite() -> unittest.TestSuite:
     names = [
+        "aiohttp",
         "annotate",
         "approxidate",
         "archive",
@@ -131,9 +132,6 @@ def self_test_suite() -> unittest.TestSuite:
         "bitmap",
         "blackbox",
         "bundle",
-        "cli",
-        "cli_cherry_pick",
-        "cli_merge",
         "client",
         "cloud_gcs",
         "commit_graph",
@@ -168,6 +166,7 @@ def self_test_suite() -> unittest.TestSuite:
         "notes",
         "objects",
         "objectspec",
+        "object_filters",
         "object_store",
         "pack",
         "patch",
@@ -179,6 +178,9 @@ def self_test_suite() -> unittest.TestSuite:
         "repository",
         "rerere",
         "server",
+        "sha256",
+        "sha256_pack",
+        "signature",
         "source",
         "sparse_patterns",
         "stash",
@@ -192,12 +194,24 @@ def self_test_suite() -> unittest.TestSuite:
         "worktree",
     ]
     module_names = ["tests.test_" + name for name in names]
+    cli_names = [
+        "cherry_pick",
+        "cli",
+        "merge",
+    ]
+    module_names += ["tests.cli.test_" + name for name in cli_names]
     porcelain_names = [
+        "annotate",
+        "bisect",
         "cherry_pick",
         "filters",
+        "ignore",
         "lfs",
+        "maintenance",
+        "mbox",
         "merge",
         "notes",
+        "rebase",
     ]
     module_names += ["tests.porcelain"] + [
         "tests.porcelain.test_" + name for name in porcelain_names
